require 'bio-hmmer3_report'
require 'forwardable'

# Top level module / namespace.
module Neurohmmer
  # A class that holds methods related to Hmmer
  class Hmmer
    class <<self
      extend Forwardable
      def_delegators Neurohmmer, :opt, :conf

      def search
        Dir.foreach(conf[:hmm_dir]) do |h|
          hmm_file = File.join(conf[:hmm_dir], h)
          next if hmm_file !~ /hmm$/
          hmm_search(opt[:input_file], hmm_file, conf[:hmm_output],
                     opt[:num_threads])
        end
      end

      def analyse_output
        hmm_results       = {}
        hmm_search_output = File.open(conf[:hmm_output])
        hmm_reports       = Bio::HMMER::HMMER3.reports(hmm_search_output)
        hmm_reports.each_with_index do |report, idx|
          next if idx + 1 == hmm_reports.length
          hmm_results[report.query] = analyse_hmm_search_report(report)
        end
        hmm_results
      end

      def generate_hmm_models
        Dir.foreach(conf[:raw_data]) do |file|
          next if file !~ /fa(sta)?$/
          np_fasta_file  = File.join(conf[:raw_data], file)
          aligned_file   = File.join(conf[:raw_alignments],
                                     "#{file.gsub(/fa(sta)?$/, '')}.aligned")
          hmm_model_file = File.join(conf[:hmm_dir],
                                     "#{file.gsub(/fa(sta)?$/, '')}.hmm")
          mafft(np_fasta_file, aligned_file, opt[:num_threads])
          hmm_build(aligned_file, hmm_model_file)
        end
      end

      private

      def analyse_hmm_search_report(report)
        report_result = []
        report.hits.each do |hit|
          seq  = Neurohmmer.extract_sequence(hit.sequence_name.strip)
          hsps = []
          hit.hsps.each { |hsp|  hsps << hsp.flatseq }
          report_result << { id: seq[0], seq: seq[1], flatseq: hsps }
        end
        report_result
      end

      def mafft(input, aligned_file, num_threads)
        `mafft --maxiterate 1000 --thread #{num_threads} '#{input}' > \
         '#{aligned_file}'`
      end

      def hmm_build(aligned_file, hmm_model_file)
        `hmmbuild '#{hmm_model_file}' '#{aligned_file}'`
      end

      def hmm_search(input_file, hmm_file, hmm_output, num_threads)
        `hmmsearch --cpu #{num_threads} '#{hmm_file}' '#{input_file}' >> \
        '#{hmm_output}'`
      end
    end
  end
end
